/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Aaron Andersen <aaron@xulplanet.com>
 *                 Manuel Reimer <Manuel.Reimer@gmx.de>
 *                 Stephen Clavering <mozilla@clav.me.uk>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

function prefbarClearCookies() {
    var cookieman = Components.classes["@mozilla.org/cookiemanager;1"]
                            .getService(Components.interfaces.nsICookieManager);
    cookieman.removeAll();
}


function prefbarClearHistory() {
    // builds from 20040210ish onward use the second class.  the relevant parts
    // of the nsIBrowserHistory are not altered though.
    var class1 = "@mozilla.org/browser/global-history;1";
    var class2 = "@mozilla.org/browser/global-history;2";
    var history = (class2 in Components.classes) ? class2 : class1;
    history = Components.classes[history].getService(Components.interfaces.nsIBrowserHistory);
    history.removeAllPages();

    // for firefox.  used to clear the history on back/forward buttons, and the
    // <browser/> session history
    var os = Components.classes["@mozilla.org/observer-service;1"]
	.getService(Components.interfaces.nsIObserverService);
    os.notifyObservers(null, "browser:purge-session-history", "");
}


function prefbarClearLocationBar() {
    // Mozilla up to 1.7 uses the class "urlbarhistory"
    var classID = Components.classes['@mozilla.org/browser/urlbarhistory;1'];
    if (classID) {
	var urlbarHistory = classID.getService(Components.interfaces.nsIUrlbarHistory);
	urlbarHistory.clearHistory();
    }
    // Mozilla since version 1.8 clears the history via RDF functions
    else {
	var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"]
	    .getService(Components.interfaces.nsIRDFService);
	var localStore = RDF.GetDataSource("rdf:local-store");
	var RDFC = Components.classes["@mozilla.org/rdf/container-utils;1"]
	    .getService(Components.interfaces.nsIRDFContainerUtils);
	var urlBarHist = RDFC.MakeSeq(localStore, RDF.GetResource("nc:urlbar-history"));
	for (var i = urlBarHist.GetCount(); i > 0; --i)
	    urlBarHist.RemoveElementAt(i, true);
    }
    navigator.preference('general.open_location.last_url', '');
}


function prefbarClearCache(aType) {
    var classID = Components.classes["@mozilla.org/network/cache-service;1"];
    var cacheService = classID.getService(Components.interfaces.nsICacheService);
    cacheService.evictEntries(aType);
}

function prefbarClearMemCache() {
    prefbarClearCache(Components.interfaces.nsICache.STORE_IN_MEMORY);
}

function prefbarClearDiskCache() {
    prefbarClearCache(Components.interfaces.nsICache.STORE_ON_DISK);
}

function prefbarClearAllCache() {
    prefbarClearMemCache();
    prefbarClearDiskCache();
}

//
// "Kill Flash" button
//

function prefbarKillFlash() {
    var frames = prefbarGetFrames(window._content.window);
    for (var frameindex in frames) {
        var page = frames[frameindex].document;

        // "Twice-Cooked Method" and <embed>

        var embeds = page.getElementsByTagName("embed");

        for(var i = embeds.length - 1; i >= 0; i--) {
            var current = embeds[i];

            if(current.type =="application/x-shockwave-flash") {
                if(current.parentNode.nodeName.toLowerCase() == "object") {
                    current = current.parentNode;
                }
                prefbarRemoveElement(page, current);
            }
        }

        // "Satay Method" and <object>

        var objects = page.getElementsByTagName("object");

        for(var i = objects.length - 1; i >= 0; i--) {
            var current = objects[i];

            if(current.type =="application/x-shockwave-flash") {
                prefbarRemoveElement(page, current);
            }
        }
    }
}

function prefbarGetFrames(owindow) {
    var winarray = new Array();
    winarray.push(owindow);
    var len = owindow.frames.length;
    for (var index = 0; index < len; index++)
        winarray = winarray.concat(prefbarGetFrames(owindow.frames[index]));
    return winarray;
}

function prefbarRemoveElement(page, element) {
    var width = element.width;
    var height = element.height;
    if (width && height) {
        var div = page.createElement("DIV");
        var text = page.createTextNode(" ");
        div.appendChild(text);

        element.parentNode.replaceChild(div, element);

        div.setAttribute("style", "height: " + height + "px; width: " + width + "px; border: 1px solid black;");
    }
    else {
        element.parentNode.removeChild(element);
    }
}

//
// Proxy serverlist
//

function prefbarSetProxy(str) {
    if (str == "") {
	navigator.preference("network.proxy.type", 0);
	/* Do we need to clear preference settings? */
    }
    else {
	var args = str.split(":");

	if (args.length != 2) {
	    alert('Wrong proxy setting: "' + str + '"');
	    return;
	}

	// Maybe we shouldn't set the Proxy Type to "Manual" automatically,
        // since some users may want to use "Proxy Serverlist" *and* "Proxy
        // Menulist" at once...
	navigator.preference("network.proxy.type", 1);

	// Set HTTP-Proxy
	navigator.preference("network.proxy.http", args[0]);
	navigator.preference("network.proxy.http_port", Number(args[1]));
	// Set SSL-Proxy
	navigator.preference("network.proxy.ssl", args[0]);
	navigator.preference("network.proxy.ssl_port", Number(args[1]));
	// Set FTP-Proxy
	navigator.preference("network.proxy.ftp", args[0]);
	navigator.preference("network.proxy.ftp_port", Number(args[1]));
	// Set Gopher-Proxy
	navigator.preference("network.proxy.gopher", args[0]);
	navigator.preference("network.proxy.gopher_port", Number(args[1]));
    }
}

function prefbarGetProxy() {
    if (navigator.preference("network.proxy.type") != 1) {
	return "";
    }

    return navigator.preference("network.proxy.http") + ":" +
	navigator.preference("network.proxy.http_port");
}

//
// "Pipelining" checkbox
//

function prefbarSetPipelining(value) {
    navigator.preference("network.http.proxy.pipelining", value);
    navigator.preference("network.http.pipelining", value);
}

function prefbarGetPipelining() {
    return (navigator.preference("network.http.proxy.pipelining") && navigator.preference("network.http.pipelining"));
}

//
// "Resize" menulist
//

function prefbarSetResolution(value) {
    var valueArray = value.split("x");
    window.outerWidth = valueArray[0];
    window.outerHeight = valueArray[1];
}

function prefbarGetResolution() {
    return window.outerWidth + "x" + window.outerHeight;
}

//
// "Images" Checkbox
//

// The mozilla developers really try to keep extension developers busy :-(
// http://kb.mozillazine.org/Permissions.default.image
// If the new preference is not undefined, then we will use this one.
// We keep on using the old one otherwise.
function prefbarSetImages(value) {
    if (navigator.preference("permissions.default.image") == undefined)
	navigator.preference("network.image.imageBehavior", value ? 0 : 2);
    else
        navigator.preference("permissions.default.image", value ? 1 : 2);
}
function prefbarGetImages() {
    if (navigator.preference("permissions.default.image") == undefined)
        return navigator.preference("network.image.imageBehavior") == 0;
    else
        return navigator.preference("permissions.default.image") == 1;
}

//
// "Save Page" button
//

var prefbarSPFilename = null;
var prefbarSPOldGetDefaultFileName = null;
var prefbarSPGetDefaultFileName = function(param1, param2, param3, param4) {
    if (prefbarSPFilename) {
	var retval = prefbarSPFilename;
	prefbarSPFilename = null;
	return retval;
    }
    else
        return prefbarSPOldGetDefaultFileName(param1, param2, param3, param4);
}

function prefbarSavePage() {
    var title = window._content.document.title;
    var uri = window._content.location;
    var doc = window._content.document;
    var rchar = "_";

    title = title.replace(/\/|\\|:|\*|\?|\"|<|>|\|/g, rchar);
    title = title.replace(/\s/g, rchar);
    if (title.match(new RegExp("^" + rchar + "+$"))) title = "";

    // Filename is the modified title
    prefbarSPFilename = title;

    if (!prefbarSPOldGetDefaultFileName) {
	if ("getDefaultFileName" in this) {
	    prefbarSPOldGetDefaultFileName = this.getDefaultFileName;
	    this.getDefaultFileName = prefbarSPGetDefaultFileName;
	}
	else {
	    alert("PrefBar: Save Page feature not available for the mozilla version you use!");
	    return;
	}
    }

    // Save function used by Mozilla up to 1.7
    if ("saveInternal" in this)
	saveInternal(uri, doc, null, false);
    // Save function used by Mozilla since 1.8 and SeaMonkey
    else if ("internalSave" in this) {
        // We want to use cached data because the document is currently visible.
        var dispHeader = null;
        try {
            dispHeader =
                doc.defaultView
                .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                .getInterface(Components.interfaces.nsIDOMWindowUtils)
                .getDocumentMetadata("content-disposition");
        } catch (ex) {
            // Failure to get a content-disposition is ok
        }

        internalSave(uri, doc, null, dispHeader,
                     doc.contentType, false, null, null);
    }
    else
        alert("PrefBar: Save Page feature not available for the mozilla version you use!");
}

//
// external application support
//

function prefbarExecute(path, arguments) {
    if (arguments == undefined) arguments = new Array;

    var executable = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    executable.initWithPath(path);

    if (!executable.exists()) {
	alert("PrefBar: Error - executable '" + path + "' not found");
	return;
    }
    try {
	var process = Components.classes["@mozilla.org/process/util;1"].createInstance(Components.interfaces.nsIProcess);

	process.init(executable);

	var pid = new Object();
	process.run(false, arguments, arguments.length, pid);
    }
    catch (ex) {
	alert("PrefBar: Error - couldn't execute '" + path + "'");
	return;
    }
}

//
// Useragent Menulist
// This code handles useragent, appname, appversion and platform spoofing
// If more than useragent is needed, then prefix the value with "js:"
// and use Javascript syntax to set the variables "useragent", "appname",
// "appversion" and "platform"
//

function prefbarSetUseragent(value) {
    var prefBranch = goPrefBar.PrefService.getBranch("");

    // This are the variables that may be changed in the short Javascript
    // in menuitems value
    var useragent;
    var appname;
    var appversion;
    var platform;

    if (value.substr(0,3) == "js:")
        eval(value.substr(3));
    else {
        // XXX Hack to get backwards compatibility
        // In new PrefBar versions the default item in an list is marked by the
        // value "PREFBARDEFAULT". This allows to have the default item
        // everywhere in the list and doesn't confuse the users (see 
        // PrefBar-Bug 5733) In old versions simply the first item was the
        // default. As we don't update the items of the "User Agent Menulist",
        // we have to be backwards compatible for this one item.
        if (value == "") value = undefined;

        // If value is PREFBARDEFAULT then we have to reset the preference
        if (value == "PREFBARDEFAULT") value = undefined;

        useragent = value;
    }

    // Either set or reset the variables that may be changed using the user
    // agent menulist.
    var pref = "general.useragent.override";
    if (useragent == undefined)
        try { prefBranch.clearUserPref(pref); } catch(e) {}
    else
        navigator.preference(pref, useragent);
    pref = "general.appname.override";
    if (appname == undefined)
        try { prefBranch.clearUserPref(pref); } catch(e) {}
    else
        navigator.preference(pref, appname);
    pref = "general.appversion.override";
    if (appversion == undefined)
        try { prefBranch.clearUserPref(pref); } catch(e) {}
    else
        navigator.preference(pref, appversion);
    pref = "general.platform.override";
    if (platform == undefined)
        try { prefBranch.clearUserPref(pref); } catch(e) {}
    else
        navigator.preference(pref, platform);
}

function prefbarGetUseragent(context) {
    var prefBranch = goPrefBar.PrefService.getBranch("");
    var defaultset = !prefBranch.prefHasUserValue("general.useragent.override");

    // XXX Hack to get backwards compatibility
    // In new PrefBar versions the default item in an list is marked by the
    // value "PREFBARDEFAULT". This allows to have the default item
    // everywhere in the list and doesn't confuse the users (see 
    // PrefBar-Bug 5733) In old versions simply the first item was the
    // default. As we don't update the items of the "User Agent Menulist",
    // we have to be backwards compatible for this one item.
    if (defaultset) {
        var defaultexists = false;
        var len = context.items.length;
        for (var index = 0; index < len; index++) {
            if (context.items[index][1] == "PREFBARDEFAULT") {
                defaultexists = true;
                break;
            }
        }

        if (defaultexists)
            context.value = "PREFBARDEFAULT";
        else
            context.value = context.items[0][1];

        return;
    }

    var uavalue = navigator.preference("general.useragent.override");
    var len = context.items.length;
    for (var index = 0; index < len; index++) {
        var curval = context.items[index][1];
        if (curval.substr(0,3) == "js:") {
            var useragent;
            var appname;
            var appversion;
            var platform;
            eval(curval.substr(3));
            if (useragent == uavalue) {
                context.value = curval;
                break;
            }
        }
        else {
            if (curval == uavalue) {
                context.value = curval;
                break;
            }
        }
    }
}

//
// Languages Menulist
// "Auto-Configures" using the settings done in the content languages GUI
// of the browser!
//

function prefbarSetLanguage(value) {
    var prefstr = goPrefBar.PrefBranch.getComplexValue("intl.accept_languages", Components.interfaces.nsIPrefLocalizedString).data;
    var langs = prefstr.split(/\s*,\s*/);
    var newlangs = Array(value);

    var len = langs.length;
    for (var index = 0; index < len; index++) {
        if (langs[index] != value)
            newlangs.push(langs[index]);
    }

    prefstr = newlangs.join(", ");
    navigator.preference("intl.accept_languages", prefstr);
}

function prefbarGetLanguage(context) {
    var prefstr = goPrefBar.PrefBranch.getComplexValue("intl.accept_languages", Components.interfaces.nsIPrefLocalizedString).data;
    var langs = prefstr.split(/\s*,\s*/);

    context.items = Array();

    var len = langs.length;
    for (var index = 0; index < len; index ++) {
        context.items.push(Array(langs[index], langs[index]));
    }

    context.value = langs[0];
}

//
// Restore Tab Button
//

var prefbarRestoreTabClosed = new Array();
var prefbarRestoreTabRemoveTab = function(aTab) {
    var max_cache = navigator.preference("extensions.prefbar.btnrestoretab.maxcache");

    if (aTab.localName != "tab")
        aTab = gBrowser.mCurrentTab;

    var browser;
    // SeaMonkey
    if ("linkedBrowser" in aTab)
        browser = aTab.linkedBrowser
    // Mozilla
    else {
        var index = -1;
        if (gBrowser.mCurrentTab == aTab)
            index = gBrowser.mPanelContainer.selectedIndex;
        else {
            for (var i = 0; i < gBrowser.mTabContainer.childNodes.length; i++)
                if (gBrowser.mTabContainer.childNodes[i] == aTab)
                    index = i;
        }
        browser = gBrowser.mPanelContainer.childNodes[index];
    }

    var position = null;
    // SeaMonkey
    if ("getTabIndex" in gBrowser) position = gBrowser.getTabIndex(aTab);
    // Firefox 1.5
    if ("_tPos" in aTab) position = aTab._tPos;
    // Mozilla: No position

    prefbarRestoreTabClosed.push(Array(position, browser.sessionHistory));
    if (prefbarRestoreTabClosed.length > max_cache)
        prefbarRestoreTabClosed.shift();
    gBrowser.prefbarOldRemoveTab(aTab);
}

function prefbarRestoreTabInit() {
    // Firefox 2.0 brings it's own restore tab backend.
    // If this exists, then exit now! We will call the internal function
    // to restore tabs, so our backend is now exclusively for SeaMonkey!
    if ("undoCloseTab" in this) {
        goPrefBar.dump("prefbarRestoreTab: Cancelled init. Firefox backend will be used!");
        return;
    }

    // For some reason, sometimes gBrowser isn't defined
    if (!gBrowser) gBrowser = document.getElementById("content");

    // MultiZilla brings its own restore tab backend.
    // If this exists, then exit now! We will call the MultiZilla function
    // to restore tabs.
    if ("restoreTab" in gBrowser) {
        goPrefBar.dump("prefbarRestoreTab: Cancelled init. MultiZilla backend will be used!");
        return;
    }

    if ("prefbarOldRemoveTab" in gBrowser) return;
    if (window.tablib) tablib.init();
    window.gBrowser.prefbarOldRemoveTab = gBrowser.removeTab;
    window.gBrowser.removeTab = prefbarRestoreTabRemoveTab;
}

function prefbarRestoreTab() {
    // Firefox 2.0 brings it's own restore tab backend.
    // If this exists, then call the internal function now and exit.
    if ("undoCloseTab" in this) {
        goPrefBar.dump("prefbarRestoreTab: Calling internal 'undoCloseTab'");
        undoCloseTab();
        return;
    }

    // MultiZilla brings its own restore tab backend.
    // If this exists, then call the MultiZilla function now and exit.
    if ("restoreTab" in gBrowser) {
        goPrefBar.dump("prefbarRestoreTab: Calling MultiZilla 'gBrowser.restoreTab'");
        gBrowser.restoreTab(0);
        return;
    }

    if (!prefbarRestoreTabClosed.length) return;
    var aTab = prefbarRestoreTabClosed.pop();
    var newTab = gBrowser.addTab("about:blank");
    if (gBrowser.moveTabTo) gBrowser.moveTabTo(newTab, aTab[0]);
    gBrowser.selectedTab = newTab;

    var oldHistory = aTab[1];
    var newHistory = gBrowser.webNavigation.sessionHistory;
    newHistory.QueryInterface(Components.interfaces.nsISHistoryInternal);

    var len = oldHistory.count;
    for (var i = 0; i < len; i++) {
        newHistory.addEntry(oldHistory.getEntryAtIndex(i, false), true);
    }

    if (oldHistory.count) gBrowser.webNavigation.gotoIndex(oldHistory.index);
}

//
// Proxies checkbox
//

function prefbarGetProxyEnabled() {
    var type = navigator.preference("network.proxy.type");
    return (type != 0);
}

function prefbarSetProxyEnabled(value) {
    var typepref = "network.proxy.type";
    var lasttypepref = "extensions.prefbar.btnproxycheck.lasttype";
    if (value == false) {
        var type = navigator.preference(typepref);
        navigator.preference(lasttypepref, type);
        navigator.preference("network.proxy.type", 0);
    }
    else {
        var lasttype = navigator.preference(lasttypepref);
        if (lasttype == 0 || lasttype == undefined) lasttype = 1;
        navigator.preference(typepref, lasttype);
    }
}

//
// Tab dependent checkboxes
//

// Javascript
function prefbarSetTabJavascript(value) {
    prefbarGetDocShell().allowJavascript = value;
}
function prefbarGetTabJavascript() {
    return prefbarGetDocShell().allowJavascript;
}
// Images
function prefbarSetTabImages(value) {
    prefbarGetDocShell().allowImages = value;
}
function prefbarGetTabImages() {
    return prefbarGetDocShell().allowImages;
}
// Plugins
function prefbarSetTabPlugins(value) {
    prefbarGetDocShell().allowPlugins = value;
}
function prefbarGetTabPlugins() {
    return prefbarGetDocShell().allowPlugins;
}
// MetaRedirects
/*
function prefbarSetTabMetaRedirect(value) {
    prefbarGetDocShell().allowMetaRedirects = value;
}
function prefbarGetTabMetaRedirect() {
    return prefbarGetDocShell().allowMetaRedirects;
}
*/

function prefbarGetDocShell() {
    var docShell = getBrowser().docShell;
    docShell.QueryInterface(Components.interfaces.nsIDocShell);
    return docShell;
}

//
// Flash Checkbox
//

function prefbarSetFlash(value) {
    // We need a hidden browser object, to do our plugin refreshs without
    // reloading pages...
    var hiddenbrowser = document.getElementById("prefbarflashhiddenbrowser");
    if (!hiddenbrowser) {
        hiddenbrowser = document.createElement("browser");
        hiddenbrowser.id = "prefbarflashhiddenbrowser";
        hiddenbrowser.setAttribute("collapsed", "true");
        var mainwin = document.getElementById("main-window");
        mainwin.appendChild(hiddenbrowser);
    }
    var PluginManager = hiddenbrowser.contentWindow.navigator.plugins;

    var PluginName = prefbarGetFlashPluginName();
    var Plugins = prefbarGetPluginPaths(PluginName);

    // No Plugin found.
    if (Plugins.length == 0) {
        alert("No Flash Plugin found!");
	return;
    }

    // More than one plugin found
    if (Plugins.length > 1) {
        var message = "The flash plugin exists in the following locations:\n\n";
        for (var pindex in Plugins)
            message = message + prefbarObj2Path(Plugins[pindex]) + "\n";

        alert(message + "\n Please delete all but one, as the Flash checkbox only works, if there is only one plugin!");
        return;
    }

    var PluginPath = prefbarObj2Path(Plugins[0]);

    // Plugin found in non-profile-path
    var pp = prefbarGetProfilePluginsPath();
    if (PluginPath != pp) {
        var warning = navigator.preference("extensions.prefbar.btnflash.dirwarning");
        if (!warning) {
            var message = "PrefBar flash checkbox\n\n";
            message += "Your flash plugin was found in the path:\n\n";
            message += PluginPath + "\n\n";
            message += "In this directory, root/admin privileges are required to toggle flash. It is strongly recommended to move the plugin to the following directory:\n\n";
            message += pp + "\n\n";
            message += "there, you don't need root/admin privileges to toggle.\n"
            message += "This message will be displayed only once, so if you really want to surf as admin, which is *BAD*!!!, then you may just ignore this message.";
            alert(message);
            navigator.preference("extensions.prefbar.btnflash.dirwarning", true);
        }
    }

    var destName = value ? PluginName : PluginName + ".prefbar";

    try {
        Plugins[0].moveTo(null, destName);
        // Only reload the plugins, if Flash has to be *disabled*, as this will
        // free the plugin and remove it from memory. Don't do this on
        // *enabling* the plugin, as this may enable Flash on pages, where
        // we don't want it!
        if (value == false)
            PluginManager.refresh(true);
        else
            PluginManager.refresh(false);
    }
    catch (e) { 
        alert("Failed to disable/enable flash plugin. Maybe you are running a flash movie in your browser, and so the plugin is locked, or you just don't have write permission to the directory '" + Plugins[0].path + "'");
    }
}

function prefbarGetFlash() {
    var Plugins = prefbarGetPluginPaths(prefbarGetFlashPluginName());

    if (Plugins.length == 0) return false;

    // True to show that Flash *may* be enabled
    if (Plugins.length > 1) return true; 

    return !Plugins[0].path.match(/\.prefbar$/);
}

function prefbarGetFlashPluginName() {
    var platform = window.navigator.platform.toLowerCase();
    if (platform.indexOf("win") != -1)
        return "npswf32.dll";
    else if (platform.indexOf("os/2") != -1)
        return "npswf2.dll";
    else
        return "libflashplayer.so";
}

// Some operating systems don't have case sensitive paths
// To be able to compare paths, any converting of filesystem objects to a path
// string is done using this function, which will convert the path to lower
// case on systems, where the upper/lowercase of filenames is just for optical
// reasons.
function prefbarObj2Path(obj) {
    var platform = window.navigator.platform.toLowerCase();
    if (platform.indexOf("win") != -1 ||
        platform.indexOf("os/2") != -1)
        return obj.path.toLowerCase();
    else
        return obj.path;
}

// Returns the plugins path inside the user profile path
// We'll have to use environment variables, to find it, as Mozilla doesn't have
// a built-in way, to find this directory.
function prefbarGetProfilePluginsPath() {
    var env = Components.classes["@mozilla.org/process/environment;1"].getService(Components.interfaces.nsIEnvironment);

    var pd = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);

    var platform = window.navigator.platform.toLowerCase();
    if (platform.indexOf("win") != -1) {
	pd.initWithPath(env.get("APPDATA"));
        pd.append("Mozilla");
        pd.append("plugins");
    }
    else {
	pd.initWithPath(env.get("HOME"));
        pd.append(".mozilla");
        pd.append("plugins");
    }

    return prefbarObj2Path(pd);
}

// We return all plugin paths (as Array of nsILocalFile objects), which either
// contain a enabled or a disabled flash plugin.
function prefbarGetPluginPaths(PluginName) {
    var PluginNames = [PluginName, PluginName + ".prefbar"];

    var plugindirs = new Array();

    // First fetch the plugin directory list to the hash array. Index ist the
    // path name, so we remove any duplicates.
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var PluginsDL = dirService.get("APluginsDL", Components.interfaces.nsISimpleEnumerator);

    while (PluginsDL.hasMoreElements()) {
        var PluginDir = PluginsDL.getNext().QueryInterface(Components.interfaces.nsILocalFile);
        plugindirs[prefbarObj2Path(PluginDir)] = PluginDir;
    }

    // On windows, we now fetch all registry-stored plugin directories to our
    // hash array
    if ("nsIWindowsRegKey" in Components.interfaces) {
        var regpaths = prefbarGetWinRegPluginPaths(PluginName);
        if (regpaths) {
            for (var index in regpaths) {
                var PluginDir = regpaths[index];
                plugindirs[prefbarObj2Path(PluginDir)] = PluginDir;
            }
        }
    }

    // Now, finally collect the paths, which contain the required plugin.
    var paths = new Array();

    for (var index in plugindirs) {
        var PluginDir = plugindirs[index];
        for (var PluginIndex in PluginNames) {
            var TmpTestDir = PluginDir.clone();
            TmpTestDir.append(PluginNames[PluginIndex]);
            if (TmpTestDir.exists())
                paths.push(TmpTestDir);
        }
    }

    return paths;
}

// Starting with Flash 9.x, Macromedia/Adobe started to use the
// "HKLM\Software\MozillaPlugins" registry key (Mozilla Bug 159445) to specify
// the path of their plugin.
// ...
// WINDOWS SUCKS!!!
function prefbarGetWinRegPluginPaths(PluginName) {
    // This function is windows only code!
    if (!("nsIWindowsRegKey" in Components.interfaces)) return false;

    var paths = new Array();

    var wrk = Components.classes["@mozilla.org/windows-registry-key;1"]
                        .createInstance(Components.interfaces.nsIWindowsRegKey);
    try {
        wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,
                 "SOFTWARE\\MozillaPlugins",
                 wrk.ACCESS_READ);
    } catch(e) { return false; }

    var len = wrk.childCount;
    for (var i=0; i < len; i++) {
        var name = wrk.getChildName(i);
        var subkey = wrk.openChild(name, wrk.ACCESS_READ);
        var fileobj = Components.classes["@mozilla.org/file/local;1"]
                                .createInstance(Components.interfaces.nsILocalFile);

        // readStringValue fails, if value doesn't exist
        // initWithPath fails, if path is invalid
        try {
            var path = subkey.readStringValue("Path");
            subkey.close();
            fileobj.initWithPath(path);
        }
        catch(e) { subkey.close(); continue; }

        if (!fileobj.exists() || !fileobj.isDirectory())
            fileobj = fileobj.parent;
        if (!fileobj.exists()) continue;
        paths.push(fileobj);
    }
    wrk.close();

    return paths;
}